package it.ATTechnologies.GSLBME.REST.Client;

import it.ATTechnologies.GSLBME.REST.Client.contrib.GSLBmeAPI;
import it.ATTechnologies.GSLBME.REST.Client.contrib.SetDynamicFQDN;

import java.io.IOException;

import javax.ws.rs.core.MediaType;

import org.kohsuke.args4j.CmdLineException;

import com.sun.jersey.api.client.ClientResponse;

public class GSLBmeRestClient
{
	public static void main(String[] args)
	{
		CMDlineParser cmdline = null;

		try
		{
			cmdline = new CMDlineParser(args);

			if (args.length == 0)
			{
				cmdline.printHelp();
				return;
			}

			if (cmdline.isShowHelp())
			{
				cmdline.printHelp();
				return;
			}

			if (cmdline.getUsername() == null || cmdline.getPassword() == null)
			{
				System.err
						.println("Username (-u) and password (-p) are mandatory");
				return;
			}

			GSLBmeAPI.login(cmdline.getUsername(), cmdline.getPassword());
		} catch (IOException e)
		{
			System.err.println(e.getMessage());
			System.exit(Constants.RETURNCODE_INTERNAL_ERROR);
		} catch (CmdLineException e)
		{
			System.err.println(e.getMessage());
			System.exit(Constants.RETURNCODE_INTERNAL_ERROR);
		}

		int returnCode = Constants.RETURNCODE_INTERNAL_ERROR;

		if (GSLBmeAPI.get() == null)
		{
			System.err.println(Constants.RETURNCODE_MSG_INTERNAL_ERROR);
			System.exit(Constants.RETURNCODE_INTERNAL_ERROR);
		}

		if (cmdline.getEncoding().equals(Constants.APIEncoding.json))
		{
			GSLBmeAPI.setEncoding(MediaType.APPLICATION_JSON);
		} else if (cmdline.getEncoding().equals(Constants.APIEncoding.xml))
		{
			GSLBmeAPI.setEncoding(MediaType.APPLICATION_XML);
		} else
		{
			System.err.println(Constants.RETURNCODE_MSG_INTERNAL_ERROR);
			System.exit(Constants.RETURNCODE_INTERNAL_ERROR);
		}

		if ((cmdline.getDynIFaceName() != null || cmdline.getDynTTLValue() != null)
				&& cmdline.getActionDynFQDN() == null)
		{
			System.err
					.println(Constants.RETURNCODE_MSG_INVALID_PARAMETERS_OPTION_MISMATCH);
			System.exit(Constants.RETURNCODE_INVALID_PARAMETERS_OPTION_MISMATCH);
		}

		// Switches the selected API method
		int howManyActions = 0;

		if (cmdline.getActionGetZoneName() != null)
			howManyActions++;

		if (cmdline.getActionGetGeohostName() != null)
			howManyActions++;

		if (cmdline.getActionGetRecordsName() != null)
			howManyActions++;

		if (cmdline.getActionDynFQDN() != null)
			howManyActions++;

		// One single method must be specified
		if (howManyActions > 1)
		{
			System.err
					.println(Constants.RETURNCODE_MSG_INVALID_PARAMETERS_MULTIPLE_ACTIONS);
			System.exit(Constants.RETURNCODE_INVALID_PARAMETERS_MULTIPLE_ACTIONS);
		} else if (howManyActions == 0)
		{
			System.err
					.println(Constants.RETURNCODE_MSG_INVALID_PARAMETERS_MISSING_ACTION);
			System.exit(Constants.RETURNCODE_INVALID_PARAMETERS_MISSING_ACTION);
		}

		ClientResponse b = null;

		if (cmdline.getActionDynFQDN() != null)
		{
			b = SetDynamicFQDN.set(cmdline);
		} else
		{
			if (cmdline.getDynIFaceName() != null)
			{
				System.err
						.println(Constants.RETURNCODE_MSG_INVALID_PARAMETERS_OPTION_MISMATCH);
				System.exit(Constants.RETURNCODE_INVALID_PARAMETERS_OPTION_MISMATCH);
			}

			if (cmdline.getActionGetZoneName() != null)
			{
				b = GSLBmeAPI.get().path("zone")
						.path(cmdline.getActionGetZoneName())
						.accept(GSLBmeAPI.getEncoding())
						.get(ClientResponse.class);
			} else if (cmdline.getActionGetGeohostName() != null)
			{
				b = GSLBmeAPI.get().path("geohost")
						.path(cmdline.getActionGetGeohostName())
						.accept(GSLBmeAPI.getEncoding())
						.get(ClientResponse.class);
			} else if (cmdline.getActionGetRecordsName() != null)
			{
				b = GSLBmeAPI.get().path("records")
						.path(cmdline.getActionGetRecordsName())
						.accept(GSLBmeAPI.getEncoding())
						.get(ClientResponse.class);
			}
		}

		if (b != null)
		{
			// Handles HTTP status code
			int httpStatus = b.getStatus();

			switch (httpStatus)
			{
			case 200:
				System.err.println(b.getEntity(String.class));
				returnCode = Constants.RETURNCODE_OK;
				break;
			case 400:
				System.err.println(b.getEntity(String.class));
				break;
			case 401:
				System.err.println(Constants.RETURNCODE_MSG_AUTH_FAILED);
				returnCode = Constants.RETURNCODE_AUTH_FAILED;
				break;
			case 404:
				System.err.println(b.getEntity(String.class));
				break;
			case 403:
				System.err.println(b.getEntity(String.class));
				break;
			case 406:
				System.err.println(b.getEntity(String.class));
				break;
			case 500:
				System.err.println(b.getEntity(String.class));
				break;
			case 503:
				System.err.println(b.getEntity(String.class));
				break;

			default:
				break;
			}
		}

		System.exit(returnCode);
	}
}
